﻿Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Linq
Imports System.Text
Imports System.IO
Imports System.Windows.Forms

''' <summary>
''' Um projeto simples para manter os dados
''' de uma coleção de figuras de carros
''' sem usar um banco de dados
''' </summary>
Public Class frmPrincipal

#Region "Declaração de Variaveis"

    Private carros As List(Of Carros)      ' Um conteiner para a coleção de carros
    Private carroAtual As Carros           ' O carro atual exibido
    Private caminhoArquivoAtual As String  ' O caminho do arquivo de dados dos carros
    Private posicaoAtual As Integer        ' A posição na lista de carros
    Private flagForm As Boolean            ' Define um estado para o formulário quando ocorrer uma mudança

#End Region

#Region "Construtor"

    Private Sub frmPrincipal_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        ' cria uma nova lista de carros
        ' pronta para escrever dados
        carros = New List(Of Carros)()
        carroAtual = New Carros()
        carros.Add(carroAtual)

        ' define a dsta de registro 
        dtpRegistro.Value = DateTime.Now

        ' inicia a posição atual
        posicaoAtual = 0

        ' marca o form como False
        flagForm = False
    End Sub


#End Region

#Region "ManutencaoCarros"
    ''' <summary>
    ''' Encerrar a Aplicação clicando na opção Sair do Menu
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SairToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SairToolStripMenuItem.Click

        If flagForm = True Then

            If MessageBox.Show(Me, "Você não salvou os dados atuais dos carros; " + _
                             " desejar salvar os dados antes de sair ? ", _
                             "Salvar dados atuais", MessageBoxButtons.YesNo) = _
                             System.Windows.Forms.DialogResult.Yes Then

                SalvarToolStripMenuItem_Click(Me, New EventArgs())
            End If
        Else
            Application.Exit()
        End If
    End Sub

    ''' <summary>
    ''' Encerrar a Aplicação clicando no ícone Sair
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbSair_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbSair.Click
        ' aciona a opção sair do menu
        SairToolStripMenuItem_Click(Me, New EventArgs())
    End Sub

    ''' <summary>
    ''' Limpa todos os campos do formulario
    ''' </summary>
    Public Sub LimpaFormulario()
        flagForm = True
        txtNome.Text = String.Empty
        cboMarca.SelectedIndex = -1
        txtModelo.Text = String.Empty
        txtDetalhes.Text = String.Empty
        dtpRegistro.Value = DateTime.Now
        picFoto.Image = Nothing
    End Sub

    ''' <summary>
    ''' Carrega imagem para o picture box
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub btnDefineImagem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDefineImagem.Click
        flagForm = True

        Dim caminhoArquivoImagem As String = String.Empty

        Dim OpenFileDialog1 As New OpenFileDialog()
        OpenFileDialog1.Title = "Abrir Arquivo Imagens"
        OpenFileDialog1.Filter = "JPEG Documents (*.jpg)|*.jpg|Gif Files|*.gif"

        If OpenFileDialog1.ShowDialog() = System.Windows.Forms.DialogResult.Cancel Then
            Return
        End If

        caminhoArquivoImagem = OpenFileDialog1.FileName

        If String.IsNullOrEmpty(caminhoArquivoImagem) Then
            Return
        End If

        If File.Exists(caminhoArquivoImagem) = False Then
            Return
        End If

        Try
            ' converte Imagem para um byte array e salva
            Dim fsImage As FileStream = Nothing
            fsImage = File.Open(caminhoArquivoImagem, FileMode.Open, FileAccess.Read)
            Dim bArrImage(fsImage.Length) As Byte
            fsImage.Read(bArrImage, 0, Convert.ToInt32(fsImage.Length))
            fsImage.Close()

            '' carroAtual representa um carro definido na classe Carros
            '' Imagem e NomeImagem são membros de Carros
            carroAtual.Imagem = bArrImage
            carroAtual.NomeImagem = caminhoArquivoImagem

            Dim ms As New MemoryStream(bArrImage)
            picFoto.Image = Image.FromStream(ms)
            ms.Dispose()

        Catch ex As Exception

            MessageBox.Show(ex.Message, "Erro ao armazenar imagem")

        End Try
    End Sub

    ''' <summary>
    ''' Cria um novo arquivo de dados para Carros
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NovoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NovoToolStripMenuItem.Click
        If flagForm = True Then
            If (MessageBox.Show(Me, "Você não salvou os dados dos carros atuais; " + _
                             "deseja salvar os dados antes de inciar um novo " + _
                             "arquivo de carros?", "Salvar dados atuais", _
                             MessageBoxButtons.YesNo) = _
                System.Windows.Forms.DialogResult.Yes) Then

                SalvarToolStripMenuItem_Click(Me, New EventArgs())
            Else
                ' descarta e inicia um novo documento
                carros = New List(Of Carros)()
                LimpaFormulario()
            End If
        Else
            ' inicia um novo documento
            carros = New List(Of Carros)()
            LimpaFormulario()
        End If
    End Sub

    ''' <summary>
    ''' Abrir um arquivo de dados de carros ja existente
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub AbrirToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AbrirToolStripMenuItem.Click

        If flagForm = True Then

            If (MessageBox.Show(Me, "Você não salvou os dados dos carros atuais, " + _
                             "deseja salvar os dados antes de abrir " + _
                             " um arquivo diferente ?", "Salvar dados atuais", _
                             MessageBoxButtons.YesNo) = _
                System.Windows.Forms.DialogResult.Yes) Then

                SalvarToolStripMenuItem_Click(Me, New EventArgs())

            Else
                Abrir()
            End If
        Else
            Abrir()
        End If
    End Sub

    ''' <summary>
    ''' Abrir um arquivo de dados para carros existente
    ''' </summary>
    Public Sub Abrir()

        Dim OpenFileDialog1 As New OpenFileDialog()
        OpenFileDialog1.Title = "Abrir arquivo de documentos para carros"
        OpenFileDialog1.InitialDirectory = "c:\dados\"
        OpenFileDialog1.Filter = "CAR Documentos (*.car)|*.car"

        If OpenFileDialog1.ShowDialog() = _
            System.Windows.Forms.DialogResult.Cancel Then
            Return
        End If

        caminhoArquivoAtual = OpenFileDialog1.FileName

        If String.IsNullOrEmpty(caminhoArquivoAtual) Then
            Return
        End If

        If File.Exists(caminhoArquivoAtual) = False Then
            Return
        End If

        carros = Serializador.Deserialize(caminhoArquivoAtual)

        ' Load bird at position zero
        If carros.Count <> 0 Then
            carroAtual = carros.ElementAt(0)
            carregaCarroAtual()
            flagForm = False
        End If

    End Sub

    ''' <summary>
    ''' Salva o arquivo de dados
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SalvarToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SalvarToolStripMenuItem.Click

        SalvaCarroAtual()

        If String.IsNullOrEmpty(caminhoArquivoAtual) Then

            Dim SaveFileDialog1 As New SaveFileDialog()

            Try
                SaveFileDialog1.Title = "Salva documento CAR"
                SaveFileDialog1.Filter = "CAR Documentos (*.car)|*.car"

                If SaveFileDialog1.ShowDialog() = _
                System.Windows.Forms.DialogResult.Cancel Then
                    Return
                End If

            Catch
                Return
            End Try

            caminhoArquivoAtual = SaveFileDialog1.FileName

        End If

        If String.IsNullOrEmpty(caminhoArquivoAtual) Then
            MessageBox.Show("O caminho não esta definido.", "Erro")
            Return
        End If

        Serializador.Serialize(caminhoArquivoAtual, carros)

        MessageBox.Show("Arquivo " + caminhoArquivoAtual + " salvo.", "Salvo")

        flagForm = False
    End Sub

    ''' <summary>
    ''' Salva o arquivo existente de carros com um novo nome
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub SalvarComoToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SalvarComoToolStripMenuItem.Click

        Dim SaveFileDialog1 As New SaveFileDialog()

        Try

            SaveFileDialog1.Title = "Salvar documento CAR"
            SaveFileDialog1.InitialDirectory = "c:\dados"
            SaveFileDialog1.Filter = "CAR Documentos (*.car)|*.car"

            If SaveFileDialog1.ShowDialog() = System.Windows.Forms.DialogResult.Cancel Then
                Return
            End If

        Catch
            Return
        End Try

        caminhoArquivoAtual = SaveFileDialog1.FileName

        If String.IsNullOrEmpty(caminhoArquivoAtual) Then
            Return
        End If

        Serializador.Serialize(caminhoArquivoAtual, carros)
        MessageBox.Show("Arquivo " + caminhoArquivoAtual + " salvo.", "Salvar Arquivo.")
        flagForm = False
    End Sub

    ''' <summary>
    ''' Salva o arquivo de dados
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbSalvar_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbSalvar.Click
        SalvarToolStripMenuItem_Click(Me, New EventArgs())
    End Sub

    ''' <summary>
    ''' Define os valores atuais do carro para o formulario
    ''' </summary>
    Private Sub SalvaCarroAtual()

        If Not String.IsNullOrEmpty(txtNome.Text) Then
            Try
                carroAtual.NomeCarro = txtNome.Text
                carroAtual.Marca = cboMarca.Text
                carroAtual.Modelo = txtModelo.Text
                carroAtual.Detalhes = txtDetalhes.Text
                carroAtual.dataRegistro = dtpRegistro.Value
                ' o byte array da imagem do carro é definida 
                ' via botão de comando
            Catch ex As Exception
                MessageBox.Show(ex.Message, "Erro")
            End Try
        End If
    End Sub

    ''' <summary>
    ''' Carrega o carro atual para o formulario
    ''' </summary>
    Private Sub carregaCarroAtual()

        Try
            txtNome.Text = carroAtual.NomeCarro
            txtDetalhes.Text = carroAtual.Detalhes
        Catch
        End Try

        Try
            cboMarca.Text = carroAtual.Marca
            txtModelo.Text = carroAtual.Modelo
        Catch
        End Try

        Try
            dtpRegistro.Value = carroAtual.dataRegistro
        Catch
        End Try

        Try
            If carroAtual.Imagem IsNot Nothing Then
                Dim ms As New MemoryStream(carroAtual.Imagem)
                picFoto.Image = Image.FromStream(ms)
                ms.Dispose()
            Else
                picFoto.Image = Nothing
            End If
        Catch
            picFoto.Image = Nothing
        End Try

    End Sub
#End Region

#Region "CRUD"
    ''' <summary>
    ''' Inclui um novo carro a lista de carros
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbIncluir_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbIncluir.Click
        SalvaCarroAtual()
        carroAtual = New Carros()
        LimpaFormulario()
        carros.Add(carroAtual)
        flagForm = True
    End Sub
    ''' <summary>
    ''' Remove o carro atual dos dados
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbExcluir_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbExcluir.Click
        carros.RemoveAt(posicaoAtual)

        If posicaoAtual = 0 Then
            posicaoAtual += 1
        Else
            posicaoAtual -= 1

            carroAtual = carros(posicaoAtual)
            carregaCarroAtual()
            flagForm = True

        End If
    End Sub
    ''' <summary>
    ''' Localiza um carro especifico
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbProcurar_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbProcurar.Click
        Dim f As New frmProcura(carros)
        AddHandler f.CarroNomeUpdated, AddressOf procuraCarro
        f.Show()
    End Sub

    ''' <summary>
    ''' Codigo para localizar carros
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ProcuraCarro(ByVal sender As Object, ByVal e As CarroNomeUpdateEventArgs)

        Dim i As Integer

        For i = 0 To carros.Count - 1

            If carros(i).NomeCarro = e.CarroNome Then
                carroAtual = carros(i)
                carregaCarroAtual()
                posicaoAtual = i
            End If

        Next

    End Sub
    Private Sub ListaDeCarrosToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ListaDeCarrosToolStripMenuItem.Click
        Dim f As New frmLista(carros)
        f.Show()
    End Sub

#End Region

#Region "Navegacao"

    ''' <summary>
    ''' Navega para o registro anterior se possivel
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    ''' 
    Private Sub tsbAnterior_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbAnterior.Click

        SalvaCarroAtual()

        If posicaoAtual <> 0 Then
            posicaoAtual -= 1
            carroAtual = carros(posicaoAtual)
            carregaCarroAtual()
        End If

    End Sub
    ''' <summary>
    ''' Navega para o próximo registro
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub tsbProximo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tsbProximo.Click
        SalvaCarroAtual()

        If posicaoAtual < (carros.Count - 1) Then
            posicaoAtual += 1
            carroAtual = carros(posicaoAtual)
            carregaCarroAtual()
        End If

    End Sub
#End Region
#Region "MudaEstado"
    ''' <summary>
    ''' Muda o estado para de FlagFrom sempre que o usuário
    ''' digitar algo no textbox nome do carro
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub txtNome_KeyPress(ByVal sender As System.Object, _
                                     ByVal e As System.Windows.Forms.KeyPressEventArgs) _
                                     Handles txtNome.KeyPress
        If Char.IsLower(e.KeyChar) Then
            'Converte o texto para caixa alta
            txtNome.SelectedText = Char.ToUpper(e.KeyChar)
            e.Handled = True
        End If

        flagForm = True

    End Sub



    ''' <summary>
    ''' Muda o estado para de FlagFrom sempre que o usuário
    ''' clicar na combo Marca
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub cboMarca_MouseClick(ByVal sender As System.Object, _
                                     ByVal e As System.Windows.Forms.MouseEventArgs) _
                                     Handles cboMarca.MouseClick

        flagForm = True

    End Sub



    ''' <summary>
    ''' Muda o estado para de FlagFrom sempre que o usuário
    ''' digitar no textbox modelo
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub txtModelo_KeyPress(ByVal sender As System.Object, _
                                     ByVal e As System.Windows.Forms.KeyPressEventArgs) _
                                     Handles txtModelo.KeyPress

        flagForm = True

    End Sub



    ''' <summary>
    ''' Muda o estado para de FlagFrom sempre que o usuário
    ''' digitar algo no campo Detalhes
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub txtDetalhes_KeyPress(ByVal sender As System.Object, _
                                     ByVal e As System.Windows.Forms.KeyPressEventArgs) _
                                     Handles txtDetalhes.KeyPress

        flagForm = True

    End Sub



    ''' <summary>
    ''' Muda o estado para de FlagFrom sempre que o usuário
    ''' sempre que o usuário abrir o datetimePicker
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub dtpRegistro_CloseUp(ByVal sender As System.Object, _
                                ByVal e As System.EventArgs) _
                                Handles dtpRegistro.CloseUp

        flagForm = True

    End Sub

#End Region

End Class
